/*
    Tucnak - VHF contest log
    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

int cfg_dsp_type=DSPT_ALSA;

struct dsp *dsp;

struct dsp *init_dsp(){
    struct dsp *dsp;

    dsp = g_new0(struct dsp, 1);   
    dsp->type = cfg_dsp_type;
    dsp->fd = -1;

    switch(dsp->type){
        case DSPT_OSS:
            dsp->setformat = oss_setformat;
            dsp->open   = oss_open;
            dsp->close  = oss_close;
            dsp->write  = oss_write;
            dsp->read   = oss_read;
            dsp->reset  = oss_reset;
            dsp->sync   = oss_sync;
            break;
        case DSPT_ALSA:
            dsp->setformat = alsa_setformat;
            dsp->open   = alsa_open;
            dsp->close  = alsa_close;
            dsp->write  = alsa_write;
            dsp->read   = alsa_read;
            dsp->reset  = alsa_reset;
            dsp->sync   = alsa_sync;
            dsp->pcm_opened = 0;
            break;
    }
    
    return dsp;
}

void free_dsp(struct dsp *dsp){
    if (!dsp) return;

    dsp->close(dsp);
    CONDGFREE(dsp->filename);
#ifdef HAVE_ALSA
    CONDGFREE(dsp->pcm_name);
#endif    
	g_free(dsp);
}



/******** OSS stuff ******************************/

int oss_open(struct dsp *dsp, int rec){
    int tmp;

    CONDGFREE(dsp->filename);
    dsp->filename=g_strdup(cfg->ssbd_dsp);

/*    dbg("open_dsp(%d)\n", rec);*/
    dsp->fd = open (cfg->ssbd_dsp, rec?O_RDONLY:O_WRONLY, 0);
    if (dsp->fd < 0){
        log_addf("Can't open file '%s'", cfg->ssbd_dsp);
        goto err;
    }

    if (cfg->ssbd_maxfrag>=2 && cfg->ssbd_maxfrag<16384){
        dsp->fragment = (cfg->ssbd_maxfrag<<16)|0x000a;
        dbg("setting fragment to 0x%08x\n", dsp->fragment);
        if (ioctl (dsp->fd, SNDCTL_DSP_SETFRAGMENT, &dsp->fragment)){
            log_addf("Can't set fragment size to 0x%x\n", dsp->fragment);
            goto err;
        }
    }   
        
    if (ioctl(dsp->fd, SNDCTL_DSP_GETFMTS, &dsp->fmt_mask)){
        log_addf("Can't get supported audio formats");
        goto err;
    }
    
    tmp=dsp->oss_format;
    if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
        log_addf("Can't set audio format %d, recommended %d", tmp, dsp->oss_format);
        
        tmp=dsp->oss_format;
        if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
            log_addf("Can't set recommended audio format %d, return %d", tmp, dsp->oss_format);
            
            tmp = dsp->oss_format = AFMT_S16_NE;
            if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
                log_addf("Can't set fallback audio %d, return %d", tmp, dsp->oss_format);
                
                dsp->oss_format = AFMT_S8;
                if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
                    log_addf("Can't set fallback audio format %d, return %d", tmp, dsp->oss_format);
                    
                    dsp->oss_format = AFMT_U8;
                    if (ioctl (dsp->fd, SNDCTL_DSP_SETFMT, &dsp->oss_format)){
                        log_addf("Can't set fallback audio format %d, return %d", tmp, dsp->oss_format);
                        goto err;
                    }
                }
            }
        }
        /* fallback ok */
    }
    

    tmp=dsp->channels;
    if (ioctl (dsp->fd, SNDCTL_DSP_CHANNELS, &dsp->channels)){
        log_addf("Can't set channels %d, recommended %d", tmp, dsp->channels);
        
        tmp=dsp->channels;
        if (ioctl (dsp->fd, SNDCTL_DSP_CHANNELS, &dsp->channels)){
            log_addf("Can't set recommended channels %d, return %d", tmp, dsp->channels);
            goto err;
        }
        /* fallback ok */
    }

    tmp=dsp->speed;
    if (ioctl (dsp->fd, SNDCTL_DSP_SPEED, &dsp->speed)){
        log_addf("Can't set samplerate %d, recommended %d", tmp, dsp->speed);
        
        tmp=dsp->speed;
        if (ioctl (dsp->fd, SNDCTL_DSP_SPEED, &dsp->speed)){
            log_addf("Can't set recommended samplerate %d, return %d", tmp, dsp->speed);
            goto err;
        }
        /* fallback ok */
    }

    if (ioctl (dsp->fd, SNDCTL_DSP_GETBLKSIZE, &dsp->blksize)){
        log_addf("Can't set block size %d", tmp, dsp->blksize);
        goto err;
    }   
    goto x;
    
err:;
    if (dsp->fd>=0) close(dsp->fd);
    dsp->fd = -1;

x:;
   /* dbg(" dsp->fd=%d\n", dsp->fd);*/
    return dsp->fd;
} 

int oss_close(struct dsp *dsp){
    
    /*ST_START;*/
    dsp->reset(dsp);
    CONDGFREE(dsp->filename);
    if (dsp->fd >= 0) {
        /*dbg("close_dsp(fd=%d)\n",dsp->fd);*/
        close(dsp->fd);
    }   
    dsp->fd=-1;
/*    ST_STOP;*/
    return 0;
}


int oss_write(struct dsp *dsp, void *data, int len){
    return write(dsp->fd, data, len);    
}


int oss_read(struct dsp *dsp, void *data, int len){
    return read(dsp->fd, data, len);    
}


int oss_reset(struct dsp *dsp){
    if (!dsp || dsp->fd<0) return 0;
    return ioctl(dsp->fd, SNDCTL_DSP_RESET, NULL); 
}


int oss_sync(struct dsp *dsp){
    if (!dsp || dsp->fd<0) return 0;
    return ioctl(dsp->fd, SNDCTL_DSP_SYNC, NULL); 
}


int oss_setformat(struct dsp *dsp, SF_INFO *sfinfo){

    switch (sfinfo->format & SF_FORMAT_SUBMASK){
/*        case SF_FORMAT_PCM_U8:
            format=AFMT_U8;
            break;
        case SF_FORMAT_PCM_S8:
            format=AFMT_S8;
            break;*/
        case SF_FORMAT_ULAW:
            dsp->oss_format=AFMT_MU_LAW;
            break;            
        case SF_FORMAT_ALAW:
            dsp->oss_format=AFMT_A_LAW;
            break;            
        case SF_FORMAT_IMA_ADPCM:
            dsp->oss_format=AFMT_IMA_ADPCM;
            break;            
        default:
            dsp->oss_format=AFMT_S16_NE;    
    }
    dsp->channels=sfinfo->channels; 
    if (dsp->channels>MAX_CHANNELS) dsp->channels=MAX_CHANNELS;
    dsp->speed=sfinfo->samplerate;   
    return 0;
}    

